@@ -19,13 +19,28 @@
 
 #undef G_DISABLE_ASSERT
 #undef G_LOG_DOMAIN
-#undef NOISY
 
 #include <string.h>
 
 #include "glib.h"
 #include "glib/gpattern.h"
 
+static gboolean noisy = FALSE;
+
+static void
+verbose (const gchar *format, ...)
+{
+  gchar *msg;
+  va_list args;
+
+  va_start (args, format);
+  msg = g_strdup_vprintf (format, args);
+  va_end (args);
+
+  if (noisy) 
+    g_print (msg);
+  g_free (msg);
+}
 
 /* keep enum and structure of gpattern.c and patterntest.c in sync */
 typedef enum
@@ -43,9 +58,11 @@ struct _GPatternSpec
   GMatchType match_type;
   guint      pattern_length;
   guint      min_length;
+  guint      max_length;
   gchar     *pattern;
 };
 
+
 static gchar *
 match_type_name (GMatchType match_type)
 {
@@ -72,12 +89,6 @@ match_type_name (GMatchType match_type)
     }
 }
 
-/* This leakes memory, but we don't care. The utf8 macro used to convert utf8 
-   back to Latin1 for feeding it to g_print. As of 2.0.1, g_print expects utf8,
-   so this is no longer necessary */
-#define utf8(str) str
-#define latin1(str) g_convert (str, -1, "UTF-8", "Latin1", NULL, NULL, NULL)
-
 static gboolean
 test_compilation (gchar *src, 
 		  GMatchType match_type, 
@@ -86,9 +97,7 @@ test_compilation (gchar *src,
 {
   GPatternSpec *spec; 
 
-#ifdef NOISY
-  g_print ("compiling \"%s\" \t", utf8(src));
-#endif
+  verbose ("compiling \"%s\" \t", src);
   spec = g_pattern_spec_new (src);
 
   if (spec->match_type != match_type)
@@ -102,8 +111,8 @@ test_compilation (gchar *src,
   if (strcmp (spec->pattern, pattern) != 0)
     {
       g_print ("failed \t(pattern: \"%s\", expected \"%s\")\n",
-	       utf8(spec->pattern),
-	       utf8(pattern));
+	       spec->pattern,
+	       pattern);
       return FALSE;
     }
   
@@ -122,12 +131,10 @@ test_compilation (gchar *src,
 	       min);
       return FALSE;
     }
-
-#ifdef NOISY  
-  g_print ("passed (%s: \"%s\")\n",
+  
+  verbose ("passed (%s: \"%s\")\n",
 	   match_type_name (spec->match_type),
 	   spec->pattern);
-#endif
   
   return TRUE;
 }
@@ -137,9 +144,7 @@ test_match (gchar *pattern,
 	    gchar *string, 
 	    gboolean match)
 {
-#ifdef NOISY
-  g_print ("matching \"%s\" against \"%s\" \t", utf8(string), utf8(pattern));
-#endif
+  verbose ("matching \"%s\" against \"%s\" \t", string, pattern);
   
   if (g_pattern_match_simple (pattern, string) != match)
     {
@@ -147,9 +152,8 @@ test_match (gchar *pattern,
       return FALSE;
     }
   
-#ifdef NOISY
-  g_print ("passed (%s)\n", match ? "match" : "nomatch");
-#endif
+  verbose ("passed (%s)\n", match ? "match" : "nomatch");
+
   return TRUE;
 }
 
@@ -162,21 +166,17 @@ test_equal (gchar *pattern1,
   GPatternSpec *p2 = g_pattern_spec_new (pattern2);
   gboolean equal = g_pattern_spec_equal (p1, p2);
 
-#ifdef NOISY
-  g_print ("comparing \"%s\" with \"%s\" \t", utf8(pattern1), utf8(pattern2));
-#endif
+  verbose ("comparing \"%s\" with \"%s\" \t", pattern1, pattern2);
 
   if (expected != equal)
     {
       g_print ("failed \t{%s, %u, \"%s\"} %s {%s, %u, \"%s\"}\n",
-	       match_type_name (p1->match_type), p1->pattern_length, utf8(p1->pattern),
+	       match_type_name (p1->match_type), p1->pattern_length, p1->pattern,
 	       expected ? "!=" : "==",
-	       match_type_name (p2->match_type), p2->pattern_length, utf8(p2->pattern));
+	       match_type_name (p2->match_type), p2->pattern_length, p2->pattern);
     }
-#ifdef NOISY
   else
-    g_print ("passed (%s)\n", equal ? "equal" : "unequal");
-#endif
+    verbose ("passed (%s)\n", equal ? "equal" : "unequal");
   
   g_pattern_spec_free (p1);
   g_pattern_spec_free (p2);
@@ -186,7 +186,7 @@ test_equal (gchar *pattern1,
 
 #define TEST_COMPILATION(src, type, pattern, min) { \
   total++; \
-  if (test_compilation (latin1(src), type, latin1(pattern), min)) \
+  if (test_compilation (src, type, pattern, min)) \
     passed++; \
   else \
     failed++; \
@@ -194,7 +194,7 @@ test_equal (gchar *pattern1,
 
 #define TEST_MATCH(pattern, string, match) { \
   total++; \
-  if (test_match (latin1(pattern), latin1(string), match)) \
+  if (test_match (pattern, string, match)) \
     passed++; \
   else \
     failed++; \
@@ -202,7 +202,7 @@ test_equal (gchar *pattern1,
 
 #define TEST_EQUAL(pattern1, pattern2, match) { \
   total++; \
-  if (test_equal (latin1(pattern1), latin1(pattern2), match)) \
+  if (test_equal (pattern1, pattern2, match)) \
     passed++; \
   else \
     failed++; \
@@ -214,6 +214,11 @@ main (int argc, char** argv)
   gint total = 0;
   gint passed = 0;
   gint failed = 0;
+  gint i;
+
+  for (i = 1; i < argc; i++) 
+      if (strcmp ("--noisy", argv[i]) == 0)
+	noisy = TRUE;
 
   TEST_COMPILATION("*A?B*", G_MATCH_ALL, "*A?B*", 3);
   TEST_COMPILATION("ABC*DEFGH", G_MATCH_ALL_TAIL, "HGFED*CBA", 8);
@@ -256,24 +261,43 @@ main (int argc, char** argv)
   TEST_MATCH("*?*x", "yx", TRUE);
   TEST_MATCH("*?*x", "xxxx", TRUE);
   TEST_MATCH("x*??", "xyzw", TRUE);
-  TEST_MATCH("*x", "\xc4x", TRUE);
-  TEST_MATCH("?x", "\xc4x", TRUE);
-  TEST_MATCH("??x", "\xc4x", FALSE);
-  TEST_MATCH("ab\xe4\xf6", "ab\xe4\xf6", TRUE);
-  TEST_MATCH("ab\xe4\xf6", "abao", FALSE);
-  TEST_MATCH("ab?\xf6", "ab\xe4\xf6", TRUE);
-  TEST_MATCH("ab?\xf6", "abao", FALSE);
-  TEST_MATCH("ab\xe4?", "ab\xe4\xf6", TRUE);
-  TEST_MATCH("ab\xe4?", "abao", FALSE);
-  TEST_MATCH("ab??", "ab\xe4\xf6", TRUE);
-  TEST_MATCH("ab*", "ab\xe4\xf6", TRUE);
-  TEST_MATCH("ab*\xf6", "ab\xe4\xf6", TRUE);
-  TEST_MATCH("ab*\xf6", "aba\xf6x\xf6", TRUE);
+  TEST_MATCH("*x", "\xc3\x84x", TRUE);
+  TEST_MATCH("?x", "\xc3\x84x", TRUE);
+  TEST_MATCH("??x", "\xc3\x84x", FALSE);
+  TEST_MATCH("ab\xc3\xa4\xc3\xb6", "ab\xc3\xa4\xc3\xb6", TRUE);
+  TEST_MATCH("ab\xc3\xa4\xc3\xb6", "abao", FALSE);
+  TEST_MATCH("ab?\xc3\xb6", "ab\xc3\xa4\xc3\xb6", TRUE);
+  TEST_MATCH("ab?\xc3\xb6", "abao", FALSE);
+  TEST_MATCH("ab\xc3\xa4?", "ab\xc3\xa4\xc3\xb6", TRUE);
+  TEST_MATCH("ab\xc3\xa4?", "abao", FALSE);
+  TEST_MATCH("ab??", "ab\xc3\xa4\xc3\xb6", TRUE);
+  TEST_MATCH("ab*", "ab\xc3\xa4\xc3\xb6", TRUE);
+  TEST_MATCH("ab*\xc3\xb6", "ab\xc3\xa4\xc3\xb6", TRUE);
+  TEST_MATCH("ab*\xc3\xb6", "aba\xc3\xb6x\xc3\xb6", TRUE);
+  TEST_MATCH("", "abc", FALSE);
+
+  TEST_MATCH("", "", TRUE);
+  TEST_MATCH("abc", "abc", TRUE);
+  TEST_MATCH("*fo1*bar", "yyyfoxfo1bar", TRUE);
+  TEST_MATCH("12*fo1g*bar", "12yyyfoxfo1gbar", TRUE);
+  TEST_MATCH("__________:*fo1g*bar", "__________:yyyfoxfo1gbar", TRUE);
+  TEST_MATCH("*abc*cde", "abcde", FALSE);
+  TEST_MATCH("*abc*cde", "abccde", TRUE);
+  TEST_MATCH("*abc*cde", "abcxcde", TRUE);
+  TEST_MATCH("*abc*?cde", "abccde", FALSE);
+  TEST_MATCH("*abc*?cde", "abcxcde", TRUE);
+  TEST_MATCH("*abc*def", "abababcdededef", TRUE);
+  TEST_MATCH("*abc*def", "abcbcbcdededef", TRUE);
+  TEST_MATCH("*acbc*def", "acbcbcbcdededef", TRUE);
+  TEST_MATCH("*a?bc*def", "acbcbcbcdededef", TRUE);
+  TEST_MATCH("*abc*def", "bcbcbcdefdef", FALSE);
+  TEST_MATCH("*abc*def*ghi", "abcbcbcbcbcbcdefefdefdefghi", TRUE);
+  TEST_MATCH("*abc*def*ghi", "bcbcbcbcbcbcdefdefdefdefghi", FALSE);
+  TEST_MATCH("_1_2_3_4_5_6_7_8_9_0_1_2_3_4_5_*abc*def*ghi", "_1_2_3_4_5_6_7_8_9_0_1_2_3_4_5_abcbcbcbcbcbcdefefdefdefghi", TRUE);
+  TEST_MATCH("fooooooo*a*bc", "fooooooo_a_bd_a_bc", TRUE);
+    
+  verbose ("\n%u tests passed, %u failed\n", passed, failed);
 
-#ifdef NOISY  
-  g_print ("\n%u tests passed, %u failed\n", passed, failed);
-#endif  
- 
   return failed;
 }
 
